@@ -6,12 +6,12 @@ A node and widget for placing data onto a grid (or not).
 
 from enum import Enum, unique
 
-from typing import Tuple, Dict, Any, List, Union, Optional
+from typing import Tuple, Dict, Any, List, Union, Optional, Sequence
 
 from plottr import QtGui, Signal, Slot, QtWidgets
 from .node import Node, NodeWidget, updateOption, updateGuiFromNode
 from ..data import datadict as dd
-from ..data.datadict import DataDict, MeshgridDataDict, DataDictBase
+from ..data.datadict import DataDict, MeshgridDataDict, DataDictBase, GriddingError
 from plottr.icons import get_gridIcon
 
 __author__ = 'Wolfgang Pfaff'
@@ -35,6 +35,9 @@ class GridOption(Enum):
     #: manually specify the shape of the grid
     specifyShape = 2
 
+    #: read the shape from DataSet Metadata (if available)
+    metadataShape = 3
+
 
 class ShapeSpecificationWidget(QtWidgets.QWidget):
     """A widget that allows the user to specify a grid shape.
@@ -48,25 +51,25 @@ class ShapeSpecificationWidget(QtWidgets.QWidget):
     #: signal that is emitted when we want to communicate a new shape
     newShapeNotification = Signal(dict)
 
-    def __init__(self, parent=None):
+    def __init__(self, parent: Optional[QtWidgets.QWidget] = None):
         super().__init__(parent)
 
-        self._axes = []
-        self._widgets = {}
+        self._axes: List[str] = []
+        self._widgets: Dict[int, Dict[str, QtWidgets.QWidget]] = {}
         self._processChanges = True
 
-        self.layout = QtWidgets.QFormLayout()
+        layout = QtWidgets.QFormLayout()
         self.confirm = QtWidgets.QPushButton('set')
-        self.layout.addRow(self.confirm)
-        self.setLayout(self.layout)
+        layout.addRow(self.confirm)
+        self.setLayout(layout)
 
         self.confirm.clicked.connect(self.signalShape)
 
-    def signalShape(self):
+    def signalShape(self) -> None:
         """When called, emit the current shape as signal"""
         self.newShapeNotification.emit(self.getShape())
 
-    def _addAxis(self, idx, name):
+    def _addAxis(self, idx: int, name: str) -> None:
         nameWidget = QtWidgets.QComboBox()
         for j, bx in enumerate(self._axes):
             nameWidget.addItem(bx)
@@ -79,13 +82,13 @@ class ShapeSpecificationWidget(QtWidgets.QWidget):
             'name': nameWidget,
             'shape': dimLenWidget,
         }
-        self.layout.insertRow(idx, nameWidget, dimLenWidget)
+        self.layout().insertRow(idx, nameWidget, dimLenWidget)
 
         nameWidget.currentTextChanged.connect(
             lambda x: self._processAxisChange(idx, x)
         )
 
-    def setAxes(self, axes: List[str]):
+    def setAxes(self, axes: List[str]) -> None:
         """Specify a set of axis dimensions
 
         If the axes do not match the previous ones, delete all
@@ -94,17 +97,17 @@ class ShapeSpecificationWidget(QtWidgets.QWidget):
         if axes != self._axes:
             self._axes = axes
 
-            for i in range(self.layout.rowCount() - 1):
+            for i in range(self.layout().rowCount() - 1):
                 self._widgets[i]['name'].deleteLater()
                 self._widgets[i]['shape'].deleteLater()
-                self.layout.removeRow(0)
+                self.layout().removeRow(0)
 
             self._widgets = {}
 
             for i, ax in enumerate(axes):
                 self._addAxis(i, ax)
 
-    def _unusedAxes(self):
+    def _unusedAxes(self) -> List[str]:
         names = self._axes.copy()
         for k, v in self._widgets.items():
             ax = v['name'].currentText()
@@ -112,12 +115,14 @@ class ShapeSpecificationWidget(QtWidgets.QWidget):
                 del names[names.index(ax)]
         return names
 
-    def _axisIndexFromName(self, name, excludeIdxs=[]):
+    def _axisIndexFromName(self, name: str,
+                           excludeIdxs: Sequence[int] = ()) -> Optional[int]:
         for k, v in self._widgets.items():
             if k not in excludeIdxs and v['name'].currentText() == name:
                 return k
+        return None
 
-    def _processAxisChange(self, idx, newName):
+    def _processAxisChange(self, idx: int, newName: str) -> None:
         if not self._processChanges:
             return
 
@@ -128,7 +133,7 @@ class ShapeSpecificationWidget(QtWidgets.QWidget):
             self._widgets[prevIdx]['name'].setCurrentText(unused[0])
             self._processChanges = True
 
-    def setShape(self, shape: Dict[str, Tuple[Union[str, int], ...]]):
+    def setShape(self, shape: Dict[str, Tuple[Union[str, int], ...]]) -> None:
         """ Set the shape, will be reflected in the values set in the widgets.
 
         :param shape: A dictionary with keys `order` and `shape`. The value
@@ -159,7 +164,7 @@ class ShapeSpecificationWidget(QtWidgets.QWidget):
 
         return {'order': tuple(order), 'shape': tuple(shape)}
 
-    def enableEditing(self, enable):
+    def enableEditing(self, enable: bool) -> None:
         for ax, widgets in self._widgets.items():
             widgets['name'].setEnabled(enable)
             widgets['shape'].setEnabled(enable)
@@ -171,7 +176,7 @@ class GridOptionWidget(QtWidgets.QWidget):
 
     optionSelected = Signal(object)
 
-    def __init__(self, parent=None):
+    def __init__(self, parent: Optional[QtWidgets.QWidget] = None):
         super().__init__(parent)
 
         self._emitUpdate = True
@@ -181,6 +186,8 @@ class GridOptionWidget(QtWidgets.QWidget):
             GridOption.noGrid: QtWidgets.QRadioButton('No grid'),
             GridOption.guessShape: QtWidgets.QRadioButton('Guess shape'),
             GridOption.specifyShape: QtWidgets.QRadioButton('Specify shape'),
+            GridOption.metadataShape: QtWidgets.QRadioButton(
+                'Read shape from metadata'),
         }
 
         btnLayout = QtWidgets.QVBoxLayout()
@@ -230,7 +237,7 @@ class GridOptionWidget(QtWidgets.QWidget):
 
         return GridOption(activeId), opts
 
-    def setGrid(self, grid: Tuple[GridOption, Dict[str, Any]]):
+    def setGrid(self, grid: Tuple[GridOption, Dict[str, Any]]) -> None:
         """Set the grid specification in the UI.
 
         :param grid: Tuple of the :class:`GridOption` and additional options.
@@ -250,7 +257,7 @@ class GridOptionWidget(QtWidgets.QWidget):
         self._emitUpdate = True
 
     @Slot(QtWidgets.QAbstractButton, bool)
-    def gridButtonSelected(self, btn: QtWidgets.QAbstractButton, checked: bool):
+    def gridButtonSelected(self, btn: QtWidgets.QAbstractButton, checked: bool) -> None:
         """Process a change in grid option radio box selection.
         Only has an effect when the change was done manually, and is not
         coming from the node.
@@ -271,13 +278,13 @@ class GridOptionWidget(QtWidgets.QWidget):
             self._emitUpdate = True
 
     @Slot()
-    def shapeSpecified(self):
+    def shapeSpecified(self) -> None:
         self.signalGridOption(self.getGrid())
 
-    def signalGridOption(self, grid):
+    def signalGridOption(self, grid: Tuple[GridOption, Dict[str, Any]]) -> None:
         self.optionSelected.emit(grid)
 
-    def setAxes(self, axes: List[str]):
+    def setAxes(self, axes: List[str]) -> None:
         """Set the available axis dimensions."""
         self.shapeSpec.setAxes(axes)
         if self.getGrid()[0] == GridOption.specifyShape:
@@ -285,11 +292,11 @@ class GridOptionWidget(QtWidgets.QWidget):
         else:
             self.enableShapeEdit(False)
 
-    def setShape(self, shape: SpecShapeType):
+    def setShape(self, shape: SpecShapeType) -> None:
         """Set the shape of the grid."""
         self.shapeSpec.setShape(shape)
 
-    def enableShapeEdit(self, enable: bool):
+    def enableShapeEdit(self, enable: bool) -> None:
         """Enable/disable shape editing"""
         self.shapeSpec.enableEditing(enable)
 
@@ -297,7 +304,7 @@ class GridOptionWidget(QtWidgets.QWidget):
 class DataGridderNodeWidget(NodeWidget):
     """Node widget for :class:`DataGridderNode`."""
 
-    def __init__(self, node: Node = None):
+    def __init__(self, node: Optional[Node] = None):
         self.icon = get_gridIcon()
         super().__init__(embedWidgetClass=GridOptionWidget)
 
@@ -312,18 +319,22 @@ class DataGridderNodeWidget(NodeWidget):
             lambda x: self.signalOption('grid')
         )
 
-    def getGrid(self):
+    def getGrid(self) -> Tuple[GridOption, Dict[str, Any]]:
+        assert self.widget is not None
         return self.widget.getGrid()
 
-    def setGrid(self, grid):
+    def setGrid(self, grid: Tuple[GridOption, Dict[str, Any]]) -> None:
+        assert self.widget is not None
         self.widget.setGrid(grid)
 
     @updateGuiFromNode
-    def setAxes(self, axes):
+    def setAxes(self, axes: List[str]) -> None:
+        assert self.widget is not None
         self.widget.setAxes(axes)
 
     @updateGuiFromNode
-    def setShape(self, shape):
+    def setShape(self, shape: Dict[str, Tuple[int, ...]]) -> None:
+        assert self.widget is not None
         self.widget.setShape(shape)
 
 
@@ -342,18 +353,18 @@ class DataGridder(Node):
 
     axesList = Signal(list)
 
-    def __init__(self, *arg, **kw):
+    def __init__(self, name: str):
 
-        self._grid = GridOption.noGrid, {}
+        self._grid: Tuple[GridOption, Dict[str, Any]] = (GridOption.noGrid, {})
         self._shape = None
         self._invalid = False
 
-        super().__init__(*arg, **kw)
+        super().__init__(name)
 
     # Properties
 
     @property
-    def grid(self):
+    def grid(self) -> Tuple[GridOption, Dict[str, Any]]:
         """Specification for how to grid the data. Consists of a main option
         and (optional) additional options.
 
@@ -368,13 +379,16 @@ class DataGridder(Node):
                 will leave tabular data as is, and flatten gridded data to result
                 in tabular data
 
-            * :attr:`GridOption.guessGrid` --
+            * :attr:`GridOption.guessShape` --
                 use :func:`.guess_shape_from_datadict` and :func:`.datadict_to_meshgrid`
                 to infer the grid, if the input data is tabular.
 
             * :attr:`GridOption.specifyShape` --
                 reshape the data using a specified shape.
 
+            * :attr:`GridOption.metadataShape` --
+                use the shape specified in the dataset metadata
+
         Some types may required additional options.
         At the moment, this is only the case for :attr:`GridOption.specifyShape`.
         Manual specification of the shape requires two additional options, `order` and `shape`:
@@ -403,7 +417,7 @@ class DataGridder(Node):
 
     @grid.setter  # type: ignore[misc]
     @updateOption('grid')
-    def grid(self, val: Tuple[GridOption, Dict[str, Any]]):
+    def grid(self, val: Tuple[GridOption, Dict[str, Any]]) -> None:
         """set the grid option. does some elementary type checking, but should
         probably be refined a bit."""
 
@@ -412,7 +426,7 @@ class DataGridder(Node):
         except TypeError:
             raise ValueError(f"Invalid grid specification.")
 
-        if not method in GridOption:
+        if method not in GridOption:
             raise ValueError(f"Invalid grid method specification.")
 
         if not isinstance(opts, dict):
@@ -422,7 +436,7 @@ class DataGridder(Node):
 
     # Processing
 
-    def validateOptions(self, data: Any):
+    def validateOptions(self, data: Any) -> bool:
         """Currently, does not perform checks beyond those of the parent class.
         """
         if not super().validateOptions(data):
@@ -430,7 +444,10 @@ class DataGridder(Node):
 
         return True
 
-    def process(self, dataIn: Optional[DataDictBase] = None):
+    def process(
+            self,
+            dataIn: Optional[DataDictBase] = None
+    ) -> Optional[Dict[str, Optional[DataDictBase]]]:
         """Process the data."""
 
         # TODO: what would be nice is to change the correct inner axis order
@@ -444,7 +461,9 @@ class DataGridder(Node):
         data = super().process(dataIn=dataIn)
         if data is None:
             return None
-        data = data['dataOut'].copy()
+        dataout = data['dataOut']
+        assert dataout is not None
+        data = dataout.copy()
         self.axesList.emit(data.axes())
 
         dout: Optional[DataDictBase] = None
@@ -452,16 +471,26 @@ class DataGridder(Node):
         order = opts.get('order', data.axes())
 
         if isinstance(data, DataDict):
-            if method is GridOption.noGrid:
+            try:
+                if method is GridOption.noGrid:
+                    dout = data.expand()
+                elif method is GridOption.guessShape:
+                    dout = dd.datadict_to_meshgrid(data)
+                elif method is GridOption.specifyShape:
+                    dout = dd.datadict_to_meshgrid(
+                        data, target_shape=opts['shape'],
+                        inner_axis_order=order,
+                    )
+                elif method is GridOption.metadataShape:
+                    dout = dd.datadict_to_meshgrid(
+                        data, use_existing_shape=True
+                    )
+            except GriddingError:
                 dout = data.expand()
-            elif method is GridOption.guessShape:
-                dout = dd.datadict_to_meshgrid(data)
-            elif method is GridOption.specifyShape:
-                dout = dd.datadict_to_meshgrid(
-                    data, target_shape=opts['shape'],
-                    inner_axis_order=order,
-                )
-
+                self.logger().info("data could not be gridded. Falling back "
+                                   "to no grid")
+                if self.ui is not None:
+                    self.ui.setGrid((GridOption.noGrid, {}))
         elif isinstance(data, MeshgridDataDict):
             if method is GridOption.noGrid:
                 dout = dd.meshgrid_to_datadict(data)
@@ -471,6 +500,10 @@ class DataGridder(Node):
                 self.logger().warning(
                     f"Data is already on grid. Ignore shape.")
                 dout = data
+            elif method is GridOption.metadataShape:
+                self.logger().warning(
+                    f"Data is already on grid. Ignore shape.")
+                dout = data
 
         else:
             self.logger().error(
@@ -481,14 +514,16 @@ class DataGridder(Node):
             return None
 
         if hasattr(dout, 'shape'):
+            assert isinstance(dout, MeshgridDataDict)
             self.shapeDetermined.emit({'order': order,
-                                       'shape': dout.shape()})  # type: ignore[attr-defined]
+                                       'shape': dout.shape()})
 
         return dict(dataOut=dout)
 
     # Setup UI
 
-    def setupUi(self):
+    def setupUi(self) -> None:
         super().setupUi()
+        assert self.ui is not None
         self.axesList.connect(self.ui.setAxes)
         self.shapeDetermined.connect(self.ui.setShape)
